import { GraphQLSchema, GraphQLFieldResolver, DocumentNode, GraphQLError, ValidationContext } from 'graphql';
import { GraphQLExtension } from 'graphql-extensions';
import { LogFunction } from './logging';
export interface GraphQLResponse {
    data?: object;
    errors?: Array<GraphQLError & object>;
    extensions?: object;
}
export interface QueryOptions {
    schema: GraphQLSchema;
    queryString?: string;
    parsedQuery?: DocumentNode;
    nonQueryError?: Error;
    rootValue?: any;
    context?: any;
    variables?: {
        [key: string]: any;
    };
    operationName?: string;
    logFunction?: LogFunction;
    validationRules?: Array<(context: ValidationContext) => any>;
    fieldResolver?: GraphQLFieldResolver<any, any>;
    formatError?: Function;
    formatResponse?: Function;
    debug?: boolean;
    tracing?: boolean;
    cacheControl?: boolean | any;
    request: Pick<Request, 'url' | 'method' | 'headers'>;
    extensions?: Array<() => GraphQLExtension>;
}
export declare function runQuery(options: QueryOptions): Promise<GraphQLResponse>;
